from datetime import datetime
from dataclasses import dataclass
from pprint import pprint

from src.generator import FakeDataGenerator


@dataclass
class User:
    user_id: int
    full_name: str
    age: int
    username: str
    email: str
    telephone: str
    join_date: datetime


user_gen = FakeDataGenerator(
    User,
    limit=10,
    mask_per_field={
        'username': 'A#2',
    },
    range_per_field={
        'age': {
            'range': [20, 25, 30],
            'frequency_distribution': [0.1, 0.3, 0.9],
        },
        'user_id': {
            'range': range(10, 100),
            'frequency_distribution': None,
        },
    },
)

if __name__ == '__main__':
    user_data_df = user_gen.load()
    user_data_df.show(n=100)
    user_data_as_dicts = user_gen.load(as_dicts=True)
    pprint(user_data_as_dicts)



from dataclasses import dataclass

from src.generator import FakeDataGenerator
from demo.user import User, user_gen


@dataclass
class Book:
    book_id: int
    author_id: int
    title: str


book_gen = FakeDataGenerator(
    Book,
    limit=10,
    foreign_keys=[{
        'self_field': 'author_id',
        'other_field': 'user_id',
        'other_model': User,
        'other_data': user_gen.load(as_dicts=True),
    }],
)


if __name__ == '__main__':
    book_df = book_gen.load()
    book_df.show()



import json

from random import choice, randint
from collections import deque

from typing import Any, Dict, Optional, List, get_type_hints

from pyspark.sql import SparkSession

from no_spark_in_my_home.src.handlers import Handler


class FakeDataGenerator:
    def __init__(
            self,
            model,
            mask_per_field=None,
            range_per_field=None,
            maxlength_per_field=None,
            foreign_keys=None,
            limit=10,
            lang='en',
            config=None,
    ):
        self.model = model
        self.config = config
        self.mask_per_field = mask_per_field
        self.range_per_field = range_per_field or {}
        self.maxlength_per_field = maxlength_per_field
        self.fields = get_type_hints(model)
        self.limit = limit
        self.lang = lang
        self._parse_foreign_keys(foreign_keys)
        self.foreign_keys = foreign_keys or {}
        self.spark = SparkSession.builder.appName('data-hack-lib').getOrCreate()
        self.json_filename = 'data.json'

        if config is not None:
            self._use_config()

    def load(
            self,
            as_json=False,
            where_clause=None,
            as_dicts=False,
    ):
        data = []
        for counter in range(1, self.limit + 1):
            item = {}
            for field_name, field_type in self.fields.items():
                handler = Handler(
                    self.lang,
                    self.mask_per_field,
                    self.range_per_field,
                    self.maxlength_per_field,
                    data,
                    self.limit,
                )
                handler.handle(item, field_name, field_type, counter)
            data.append(item)
        self._handle_foreign_key_relations(data)
        self._save_to_json(json.dumps(data))
        df = self.spark.read.json(self.json_filename)
        df = self._parse_where_clause(where_clause) or df
        if as_json:
            return json.dumps(data)
        if as_dicts:
            data = []
            for item in df.toJSON().collect():
                data.append(json.loads(item))
            return data
        return df

    def _use_config(self):
        with open(self.config, 'r') as config:
            json_config = json.loads(config.read())
            for model_name, model_values in json_config.items():
                if model_name.lower() == self.model.__name__.lower():
                    self.limit = model_values.get('limit', self.limit)
                    self.range_per_field = model_values.get('range_per_field', self.range_per_field)
                    self.mask_per_field = model_values.get('mask_per_field', self.mask_per_field)
                    self.maxlength_per_field = model_values.get('maxlength_per_field', self.maxlength_per_field)

    def _save_to_json(self, data):
        with open(self.json_filename, 'w') as json_file:
            json_file.write(data)

    def _parse_where_clause(self, where_clause: Optional[str]):
        if where_clause is not None:
            df = self.spark.read.json(self.json_filename)
            df = df.where(where_clause)
            return df

    def _parse_foreign_keys(self, foreign_keys: Optional[List[Dict[str, Any]]]):
        if foreign_keys is not None:
            for foreign_key in foreign_keys:
                self_field = foreign_key['self_field']
                other_field = foreign_key['other_field']
                other_data = foreign_key['other_data']
                other_model = foreign_key['other_model']
                self._check_self_field_exists(self_field)
                self._check_other_field_exists(other_field, other_data, other_model)

    @staticmethod
    def _check_other_field_exists(other_field, other_data, other_model):
        for item in other_data:
            if other_field not in item.keys():
                raise AttributeError(f'There is not such field {other_field} in {other_model.__name__}')

    def _check_self_field_exists(self, self_field):
        if self_field not in self.fields.keys():
            raise AttributeError(f'There is not such field {self_field} in {self.model.__name__} {self.fields.keys()}')

    def _handle_single_foreign_key(self, foreign_key_desc: dict, self_data):
        other_model_field_values = []
        self_field = foreign_key_desc['self_field']
        other_field = foreign_key_desc['other_field']
        other_data = foreign_key_desc['other_data']
        other_model = foreign_key_desc['other_model']
        other_model = other_model.__name__
        allow_other_has_no_refs = foreign_key_desc.get('allow_other_has_no_refs')

        for row in other_data:
            other_key_value = row[other_field]
            other_model_field_values.append(other_key_value)
        
        if allow_other_has_no_refs:
            for row in self_data:
                key_value = choice(other_model_field_values)
                row[self_field] = key_value
        else:
            other_model_field_values = deque(other_model_field_values)
            shift = randint(1, 100)
            other_model_field_values.rotate(shift)
            for idx, row in enumerate(self_data):
                key_value = other_model_field_values[idx]
                row[self_field] = key_value

    def _handle_foreign_key_relations(self, self_data):
        for key in self.foreign_keys:
            self._handle_single_foreign_key(key, self_data)



import random
import datetime

import pandas as pd

from mimesis import Person, Address, Datetime, Text
from mimesis.locales import Locale


class BaseHandler:
    def __init__(
            self,
            lang,
            mask_per_field,
            range_per_field,
            maxlength_per_field,
            data,
            limit,
    ):
        self.lang = lang.upper()
        self.limit = limit
        self.data = data
        self.range_per_field = range_per_field
        self.mask_per_field = mask_per_field or {}
        self.maxlength_per_field = maxlength_per_field or {}

        self.choices_per_field = {}

    def handle(self, item, field_name, field_type, counter):
        raise NotImplemented

    def _set_choices_per_field(self, field_name, item_range, item_frequency_distribution):
        self.choices_per_field[field_name] = random.choices(
            population=item_range,
            weights=item_frequency_distribution,
            k=self.limit,
        )

    def _process_int_range(self, item, field_name, default_item_range=range(0, 1000)):
        if field_name in self.range_per_field.keys():
            item_range = self.range_per_field[field_name]['range']
            item_frequency_distribution = self.range_per_field[field_name].get('frequency_distribution')
            if item_frequency_distribution:
                if field_name not in self.choices_per_field:
                    self._set_choices_per_field(field_name, item_range, item_frequency_distribution)
                item[field_name] = self.choices_per_field[field_name].pop(0)
            else:
                item[field_name] = random.randint(item_range[0], item_range[-1])
        else:
            item[field_name] = random.randint(default_item_range[0], default_item_range[-1])

    def _process_text_range(self, item, field_name):
        if field_name in self.range_per_field.keys():
            item_range = self.range_per_field[field_name]['range']
            item_frequency_distribution = self.range_per_field[field_name].get('frequency_distribution')
            if item_frequency_distribution:
                if field_name not in self.choices_per_field:
                    self._set_choices_per_field(field_name, item_range, item_frequency_distribution)
                item[field_name] = self.choices_per_field[field_name].pop(0)
            else:
                item[field_name] = random.choice(item_range)

    def _process_maxlength(self, item, field_name):
        for field in self.maxlength_per_field:
            if field_name == field['field_name']:
                maxlength = field['maxlength']
                allowed_symbols = field.get('allowed_symbols', 'A')
                if field['fixed']:
                    if len(item[field_name]) < maxlength:
                        shift = maxlength - len(item[field_name])
                        item[field_name] += random.choice(allowed_symbols) * shift
                if len(item[field_name]) > maxlength:
                    item[field_name] = item[field_name][:maxlength]

    def _process_mask(self, item, field_name):
        if field_name in self.mask_per_field.keys():
            mask = self.mask_per_field[field_name]
            fixed_symbols_ids = []
            for char_id, char in enumerate(mask):
                if char != '#':
                    fixed_symbols_ids.append({
                        'id': char_id,
                        'symbol': char,
                    })
            for fixed_symbol in fixed_symbols_ids:
                original_type = type(item[field_name])
                new_string = list(str(item[field_name]))
                for char_id, _ in enumerate(str(item[field_name])):
                    if char_id == fixed_symbol['id']:
                        new_string[char_id] = fixed_symbol['symbol']
                new_string = ''.join(new_string)
                if original_type is not str:
                    new_string = original_type(new_string)
                item[field_name] = new_string


class Handler(BaseHandler):
    def handle(self, item, field_name, field_type, counter):
        for handler_cls in BaseHandler.__subclasses__():
            if handler_cls is not self.__class__:
                handler = handler_cls(
                    self.lang,
                    self.mask_per_field,
                    self.range_per_field,
                    self.maxlength_per_field,
                    self.data,
                    self.limit,
                )
                handler.handle(item, field_name, field_type, counter)
        if field_name not in item:
            text_generator = Text(eval(f'Locale.{self.lang}'))
            item[field_name] = text_generator.sentence()
            self._process_maxlength(item, field_name)
            self._process_mask(item, field_name)


class MimesisPersonProviderHandler(BaseHandler):
    def handle(self, item, field_name, field_type, counter):
        keywords = (
            'full_name',
            'last_name',
            'first_name',
            'username',
            'email',
            'telephone',
        )
        for keyword in keywords:
            if keyword in field_name:
                person = Person(eval(f'Locale.{self.lang}'))
                try:
                    item[field_name] = eval(f'person.{keyword}()')
                except (TypeError, AttributeError):
                    item[field_name] = eval(f'person.{keyword}()')
                self._process_maxlength(item, field_name)
                self._process_mask(item, field_name)


class MimesisAddressProviderHandler(BaseHandler):
    def handle(self, item, field_name, field_type, counter):
        keywords = (
            'city',
            'address',
            'country',
            'country_code',
            'building_number',
            'street_address',
            'street_name',
        )
        for keyword in keywords:
            if keyword in field_name:
                address = Address(eval(f'Locale.{self.lang}'))
                item[field_name] = eval(f'address.{keyword}()')
                self._process_maxlength(item, field_name)
                self._process_mask(item, field_name)


class MimesisDatetimeProviderHandler(BaseHandler):
    def handle(self, item, field_name, field_type, counter):
        keywords = (
            'century',
            'date',
            'time',
            'datetime',
            'month',
            'timezone',
            'timestamp',
        )
        datetime_synonyms = (
            'join_date',
            'created',
            'updated',
            'date_of_birth',
            'joined',
        )
        for keyword in (*keywords, *datetime_synonyms):
            if keyword in field_name:
                date_time = Datetime(eval(f'Locale.{self.lang}'))
                if keyword in datetime_synonyms:
                    item[field_name] = date_time.datetime()
                else:
                    item[field_name] = eval(f'date_time.{keyword}()')


class IntHandler(BaseHandler):
    def handle(self, item, field_name, field_type, counter):
        if field_type is int:
            self._process_int_range(item, field_name)
            self._process_mask(item, field_name)


class DateTimeHandler(BaseHandler):
    def handle(self, item, field_name, field_type, counter):
        if field_type in (datetime.datetime, datetime.date, 'datetime', 'date'):
            if field_name in self.range_per_field.keys():
                item_range = self.range_per_field[field_name]['range']
                item_frequency_distribution = self.range_per_field[field_name].get('frequency_distribution')
                if item_frequency_distribution:
                    if field_name not in self.choices_per_field:
                        self._set_choices_per_field(field_name, item_range, item_frequency_distribution)
                    item[field_name] = self.choices_per_field[field_name].pop(0)
                else:
                    start_date = self.range_per_field[field_name]['range'][0]
                    end_date = self.range_per_field[field_name]['range'][-1]
                    item[field_name] = random.choice(pd.date_range(start_date, end_date).tolist())

            if field_name in item:
                item[field_name] = item[field_name].strftime('%d/%m/%Y, %H:%M:%S')


class IDHandler(BaseHandler):
    def handle(self, item, field_name, field_type, counter):
        if 'id' in field_name:
            if self.data:
                ids = []
                for obj in self.data:
                    ids.append(obj[field_name])
                item[field_name] = max(ids) + 1
            else:
                if field_name in self.range_per_field.keys():
                    counter = self.range_per_field[field_name]['range'][0]
                item[field_name] = counter


class AgeHandler(BaseHandler):
    def handle(self, item, field_name, field_type, counter):
        if 'age' in field_name:
            self._process_int_range(item, field_name, (1, 100))
            self._process_mask(item, field_name)


class TitleHandler(BaseHandler):
    def handle(self, item, field_name, field_type, counter):
        if 'title' in field_name:
            text = Text(eval(f'Locale.{self.lang}')).sentence()
            new_text = []
            for word in text.split():
                new_text.append(word)
                if '.' in word:
                    break
            item[field_name] = ' '.join(new_text)
            self._process_maxlength(item, field_name)
            self._process_mask(item, field_name)


class TextHandler(BaseHandler):
    def handle(self, item, field_name, field_type, counter):
        if field_type is str and field_name in self.range_per_field.keys():
            self._process_text_range(item, field_name)
            self._process_maxlength(item, field_name)
            self._process_mask(item, field_name)



#!/usr/bin/env python

from distutils.core import setup

from setuptools import setup, find_packages

setup(
    name='no_spark_in_my_home',
    version='1.0.4',
    author=['scripthound, nomilkinmyhome'],
    install_requires=[
        'faker',
        'pyspark',
        'pandas',
    ],
    setup_requires=['wheel', 'mimesis', 'pyspark', 'pandas'],
    packages=find_packages()
)